{$APPTYPE CONSOLE}
program UnRdll;
(* ported to Delphi by Eugene Kotlyarov          *)
(* fido: 2:5058/26.9 email: eugenek@mail.esoo.ru *)

(* Little modified by Sergei Sotnik                              *)
(* EMail: sergei@alice.dp.ua   http://alice.dp.ua/ppages/sotnik/ *)

uses Windows,Unrar,SysUtils;

// enum { EXTRACT, TEST, PRINT };
const
  EXTRACT = 0;
  TEST    = 1;
  PRINT   = 2;

  CRLF = #13#10;

procedure ShowComment(CmtBuf : PChar);
begin
  writeln(CRLF,'Comment:',CRLF,CmtBuf,CRLF);
end;

procedure OutHelp;
begin
  write(#13#10,'UNRDLL.   This is a simple example of UNRAR.DLL usage'#13#10);
  write(#13#10'Syntax:'#13#10);
  write(#13#10'UNRDLL X <Archive>     extract archive contents');
  write(#13#10'UNRDLL T <Archive>     test archive contents');
  write(#13#10'UNRDLL P <Archive>     print archive contents to stdout');
  write(#13#10'UNRDLL L <Archive>     view archive contents'#13#10);
end;

function ChangeVolProc(ArcName:PChar;Mode:integer):integer; cdecl export;
var buf: array[0..15] of char;
    nr, cMode: cardinal;
  function FileToHandle(const F: Text): THandle;
  type
    PHandle=^THandle;
  var h: PHandle;
  begin
    h := @f; // first 4 bytes of File record is it's Windows Handler
    result := h^;
  end;
begin
  Writeln('Start change');
  GetConsoleMode(FileToHandle(input), cmode);
  SetConsoleMode(FileToHandle(input), ENABLE_PROCESSED_INPUT);
  if (Mode = RAR_VOL_ASK) then
  begin
    write(#13#10'Insert disk with ',ArcName,' and press ''Enter'' or enter''Q'' to exit ');
    ReadConsole( FileToHandle(input) , @buf[0], 1, nr, nil);
    SetConsoleMode(FileToHandle(input), cmode);
    result:=integer((UpCase(buf[0])<>'Q'));
    Exit;
  end;
  result:=1;
end;

function ProcessDataProc(Addr:PUCHAR;Size:integer):integer; cdecl export;
begin
  Flush(Output);
  FileWrite(TTextRec(Output).Handle,Addr^,Size);
  Flush(Output);
  result:=1;
end;

procedure OutOpenArchiveError(Error:integer; ArcName:PChar);
begin
  case Error of
    ERAR_NO_MEMORY:  write(#13#10'Not enough memory');
    ERAR_EOPEN:      write(#13#10'Cannot open ',ArcName);
    ERAR_BAD_ARCHIVE:write(#13#10,ArcName,' is not RAR archive');
    ERAR_BAD_DATA:   write(#13#10,ArcName,': archive header broken');
  end;
end;

procedure OutProcessFileError(Error:integer);
begin
  case (Error) of
    ERAR_UNKNOWN_FORMAT:write('Unknown archive format');
    ERAR_BAD_ARCHIVE:   write('Bad volume');
    ERAR_ECREATE:       write('File create error');
    ERAR_EOPEN:         write('Volume open error');
    ERAR_ECLOSE:        write('File close error');
    ERAR_EREAD:         write('Read error');
    ERAR_EWRITE:        write('Write error');
    ERAR_BAD_DATA:      write('CRC error');
  end;
end;


procedure ListArchive(ArcName : PChar);
var
  hArcData:THandle;
  RHCode,PFCode:integer;
  CmtBuf:array [0..16383] of char;
  HeaderData: RARHeaderData;
  OpenArchiveData:RAROpenArchiveData;

begin
  OpenArchiveData.ArcName:=ArcName;
  OpenArchiveData.CmtBuf:=CmtBuf;
  OpenArchiveData.CmtBufSize:=sizeof(CmtBuf);
  OpenArchiveData.OpenMode:=RAR_OM_LIST;
  hArcData:=RAROpenArchive(OpenArchiveData);

  if (OpenArchiveData.OpenResult <> 0) then
  begin
    OutOpenArchiveError(OpenArchiveData.OpenResult,ArcName);
    Exit;
  end;

  if (OpenArchiveData.CmtState = 1) then ShowComment(CmtBuf);

  RARSetChangeVolProc(hArcData,ChangeVolProc);

  HeaderData.CmtBuf:=CmtBuf;
  HeaderData.CmtBufSize:=sizeof(CmtBuf);

  write(#13#10'File                       Size');
  write(#13#10'-------------------------------');
  RHCode:=RARReadHeader(hArcData,  HeaderData);
  while (RHCode = 0) do
  begin
//    printf("\n%-20s %10d ",HeaderData.FileName,HeaderData.UnpSize);
    write(#13#10,HeaderData.FileName,' ',HeaderData.UnpSize,' ');

    if (HeaderData.CmtState = 1) then
      ShowComment(CmtBuf);

    PFCode:=RARProcessFile(hArcData,RAR_SKIP,nil,nil);
    if (PFCode<>0) then
    begin
      OutProcessFileError(PFCode);
      break;
    end;
    RHCode:=RARReadHeader(hArcData,  HeaderData)
  end;

  if (RHCode = ERAR_BAD_DATA) then
    write(#13#10'File header broken');

  RARCloseArchive(hArcData);
end;

procedure ExtractArchive(ArcName:PChar;Mode:integer);
var
  hArcData:THandle;
  RHCode,PFCode :integer;
  CmtBuf:array [0..16384] of char;
  HeaderData: RARHeaderData;
  OpenArchiveData: RAROpenArchiveData;
begin
  OpenArchiveData.ArcName:=ArcName;
  OpenArchiveData.CmtBuf:=CmtBuf;
  OpenArchiveData.CmtBufSize:=sizeof(CmtBuf);
  OpenArchiveData.OpenMode:=RAR_OM_EXTRACT;
  hArcData:=RAROpenArchive(OpenArchiveData);

  if (OpenArchiveData.OpenResult<>0) then
  begin
    OutOpenArchiveError(OpenArchiveData.OpenResult,ArcName);
    exit;
  end;

  if (OpenArchiveData.CmtState = 1) then
    ShowComment(CmtBuf);

  RARSetChangeVolProc(hArcData,ChangeVolProc);

  if (Mode = PRINT) then
    RARSetProcessDataProc(hArcData,ProcessDataProc);

  HeaderData.CmtBuf:=nil;

  RHCode:=RARReadHeader(hArcData,HeaderData);
  while (RHCode = 0) do
  begin
    case (Mode) of
      EXTRACT: write(#13#10'Extracting ',HeaderData.FileName:45);
      TEST:    write(#13#10'Testing ',HeaderData.FileName:45);
      PRINT:   writeln(#13#10'Printing ',HeaderData.FileName:45);
    end;

    if (Mode = EXTRACT) then
      PFCode:=RARProcessFile(hArcData,RAR_EXTRACT,nil,nil)
    else
      PFCode:=RARProcessFile(hArcData,RAR_TEST,nil,nil);

    if (PFCode = 0) then
      write(' Ok')
    else
    begin
      OutProcessFileError(PFCode);
      break;
    end;
    RHCode:=RARReadHeader(hArcData,HeaderData);
  end; // while

  if (RHCode = ERAR_BAD_DATA) then write(#13#10'File header broken');

  RARCloseArchive(hArcData);

end;

begin
  Assign (input,'');
  Reset (input);
  Assign (output,'');
  Rewrite (output);

  if (ParamCount<2) then
  begin
    OutHelp;
    Halt(0);
  end;

  case UpCase(ParamStr(1)[1]) of
    'X':ExtractArchive(PChar(ParamStr(2)),EXTRACT);
    'T':ExtractArchive(PChar(ParamStr(2)),TEST);
    'P':ExtractArchive(PChar(ParamStr(2)),PRINT);
    'L':ListArchive(PChar(ParamStr(2)));
  else begin
      OutHelp;
      Halt(0);
    end;
  end;
  Halt(0);
end.
